fn fuel_for(mass: u64) -> u64 {
    (mass / 3).saturating_sub(2)
}

fn total_fuel(mass: u64) -> u64 {
    std::iter::successors(Some(fuel_for(mass)), |&fuel| Some(fuel_for(fuel)))
        .take_while(|&fuel| fuel > 0)
        .sum()
}

pub fn main(masses: Vec<u64>) {
    let total: u64 = masses.into_iter().map(total_fuel).sum();
    println!("Fuel needed for bringing Santa home is: {}", total);
}

#[test]
fn test_part1() {
    assert_eq!(fuel_for(12), 2);
    assert_eq!(fuel_for(14), 2);
    assert_eq!(fuel_for(1969), 654);
    assert_eq!(fuel_for(100756), 33583);
}

#[test]
fn test_part2() {
    assert_eq!(total_fuel(14), 2);
    assert_eq!(total_fuel(1969), 966);
    assert_eq!(total_fuel(100756), 50346);
}
